@@ -6,9 +6,6 @@
 // build-pass (FIXME(62277): could be check-pass?)
 // edition:2018
 
-#![feature(async_await)]
-#![allow(unused)]
-
 async fn foo<F>(_: &(), _: F) {}
 
 fn main() {
