@@ -5,11 +5,11 @@ use std::str;
 
 use serde::Serialize;
 
-use crate::support::cargo_process;
-use crate::support::git;
-use crate::support::paths;
-use crate::support::registry::{cksum, Package};
-use crate::support::{basic_manifest, project, ProjectBuilder};
+use cargo_test_support::cargo_process;
+use cargo_test_support::git;
+use cargo_test_support::paths;
+use cargo_test_support::registry::{cksum, Package};
+use cargo_test_support::{basic_manifest, project, t, ProjectBuilder};
 
 fn setup() {
     let root = paths::root();
@@ -73,7 +73,7 @@ impl VendorPackage {
     }
 }
 
-#[test]
+#[cargo_test]
 fn simple() {
     setup();
 
@@ -112,7 +112,7 @@ fn simple() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn simple_install() {
     setup();
 
@@ -141,18 +141,20 @@ fn simple_install() {
 
     cargo_process("install bar")
         .with_stderr(
-            "  Installing bar v0.1.0
-   Compiling foo v0.0.1
-   Compiling bar v0.1.0
-    Finished release [optimized] target(s) in [..]s
-  Installing [..]bar[..]
-warning: be sure to add `[..]` to your PATH to be able to run the installed binaries
+            "\
+[INSTALLING] bar v0.1.0
+[COMPILING] foo v0.0.1
+[COMPILING] bar v0.1.0
+[FINISHED] release [optimized] target(s) in [..]s
+[INSTALLING] [..]bar[..]
+[INSTALLED] package `bar v0.1.0` (executable `bar[EXE]`)
+[WARNING] be sure to add `[..]` to your PATH to be able to run the installed binaries
 ",
         )
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn simple_install_fail() {
     setup();
 
@@ -196,7 +198,7 @@ required by package `bar v0.1.0`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn install_without_feature_dep() {
     setup();
 
@@ -229,18 +231,20 @@ fn install_without_feature_dep() {
 
     cargo_process("install bar")
         .with_stderr(
-            "  Installing bar v0.1.0
-   Compiling foo v0.0.1
-   Compiling bar v0.1.0
-    Finished release [optimized] target(s) in [..]s
-  Installing [..]bar[..]
-warning: be sure to add `[..]` to your PATH to be able to run the installed binaries
+            "\
+[INSTALLING] bar v0.1.0
+[COMPILING] foo v0.0.1
+[COMPILING] bar v0.1.0
+[FINISHED] release [optimized] target(s) in [..]s
+[INSTALLING] [..]bar[..]
+[INSTALLED] package `bar v0.1.0` (executable `bar[EXE]`)
+[WARNING] be sure to add `[..]` to your PATH to be able to run the installed binaries
 ",
         )
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn not_there() {
     setup();
 
@@ -277,7 +281,7 @@ required by package `foo v0.1.0 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn multiple() {
     setup();
 
@@ -323,7 +327,7 @@ fn multiple() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn crates_io_then_directory() {
     let p = project()
         .file(
@@ -380,7 +384,7 @@ fn crates_io_then_directory() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn crates_io_then_bad_checksum() {
     let p = project()
         .file(
@@ -427,7 +431,7 @@ unable to verify that `bar v0.1.0` is the same as when the lockfile was generate
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_file_checksum() {
     setup();
 
@@ -471,7 +475,7 @@ the source
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn only_dot_files_ok() {
     setup();
 
@@ -503,7 +507,7 @@ fn only_dot_files_ok() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn random_files_ok() {
     setup();
 
@@ -536,13 +540,12 @@ fn random_files_ok() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn git_lock_file_doesnt_change() {
     let git = git::new("git", |p| {
         p.file("Cargo.toml", &basic_manifest("git", "0.5.0"))
             .file("src/lib.rs", "")
-    })
-    .unwrap();
+    });
 
     VendorPackage::new("git")
         .file("Cargo.toml", &basic_manifest("git", "0.5.0"))
@@ -606,7 +609,7 @@ fn git_lock_file_doesnt_change() {
     assert_eq!(lock1, lock2, "lock files changed");
 }
 
-#[test]
+#[cargo_test]
 fn git_override_requires_lockfile() {
     VendorPackage::new("git")
         .file("Cargo.toml", &basic_manifest("git", "0.5.0"))
@@ -664,7 +667,7 @@ restore the source replacement configuration to continue the build
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn workspace_different_locations() {
     let p = project()
         .no_manifest()
@@ -722,7 +725,7 @@ fn workspace_different_locations() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn version_missing() {
     setup();
 
