@@ -1,9 +1,7 @@
 use std::fs::File;
 
-use git2;
-
-use crate::support::git;
-use crate::support::{basic_manifest, project};
+use cargo_test_support::git;
+use cargo_test_support::{basic_manifest, clippy_is_available, is_nightly, project};
 
 use std::io::Write;
 
@@ -28,7 +26,7 @@ fn do_not_fix_broken_builds() {
     p.cargo("fix --allow-no-vcs")
         .env("__CARGO_FIX_YOLO", "1")
         .with_status(101)
-        .with_stderr_contains("[ERROR] Could not compile `foo`.")
+        .with_stderr_contains("[ERROR] could not compile `foo`.")
         .run();
     assert!(p.read_file("src/lib.rs").contains("let mut x = 3;"));
 }
@@ -141,8 +139,7 @@ fn broken_fixes_backed_out() {
         .env("__CARGO_FIX_YOLO", "1")
         .env("RUSTC", p.root().join("foo/target/debug/foo"))
         .with_stderr_contains(
-            "\
-             warning: failed to automatically apply fixes suggested by rustc \
+            "warning: failed to automatically apply fixes suggested by rustc \
              to crate `bar`\n\
              \n\
              after fixes were automatically applied the compiler reported \
@@ -413,7 +410,7 @@ fn specify_rustflags() {
 [FINISHED] [..]
 ";
     p.cargo("fix --edition --allow-no-vcs")
-        .env("RUSTFLAGS", "-C target-cpu=native")
+        .env("RUSTFLAGS", "-C linker=cc")
         .with_stderr(stderr)
         .with_stdout("")
         .run();
@@ -517,8 +514,7 @@ fn preserve_line_endings() {
     let p = project()
         .file(
             "src/lib.rs",
-            "\
-             fn add(a: &u32) -> u32 { a + 1 }\r\n\
+            "fn add(a: &u32) -> u32 { a + 1 }\r\n\
              pub fn foo() -> u32 { let mut x = 3; add(&x) }\r\n\
              ",
         )
@@ -535,9 +531,8 @@ fn fix_deny_warnings() {
     let p = project()
         .file(
             "src/lib.rs",
-            "\
-                #![deny(warnings)]
-                pub fn foo() { let mut x = 3; drop(x); }
+            "#![deny(warnings)]
+             pub fn foo() { let mut x = 3; drop(x); }
             ",
         )
         .build();
@@ -703,8 +698,7 @@ fn warns_if_no_vcs_detected() {
     p.cargo("fix")
         .with_status(101)
         .with_stderr(
-            "\
-             error: no VCS found for this package and `cargo fix` can potentially perform \
+            "error: no VCS found for this package and `cargo fix` can potentially perform \
              destructive changes; if you'd like to suppress this error pass `--allow-no-vcs`\
              ",
         )
@@ -714,15 +708,8 @@ fn warns_if_no_vcs_detected() {
 
 #[cargo_test]
 fn warns_about_dirty_working_directory() {
-    let p = project().file("src/lib.rs", "pub fn foo() {}").build();
+    let p = git::new("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
 
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
     File::create(p.root().join("src/lib.rs")).unwrap();
 
     p.cargo("fix")
@@ -745,15 +732,8 @@ commit the changes to these files:
 
 #[cargo_test]
 fn warns_about_staged_working_directory() {
-    let p = project().file("src/lib.rs", "pub fn foo() {}").build();
+    let (p, repo) = git::new_repo("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
 
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
     File::create(&p.root().join("src/lib.rs"))
         .unwrap()
         .write_all("pub fn bar() {}".to_string().as_bytes())
@@ -780,33 +760,17 @@ commit the changes to these files:
 
 #[cargo_test]
 fn does_not_warn_about_clean_working_directory() {
-    let p = project().file("src/lib.rs", "pub fn foo() {}").build();
-
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
-
+    let p = git::new("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
     p.cargo("fix").run();
 }
 
 #[cargo_test]
 fn does_not_warn_about_dirty_ignored_files() {
-    let p = project()
-        .file("src/lib.rs", "pub fn foo() {}")
-        .file(".gitignore", "bar\n")
-        .build();
+    let p = git::new("foo", |p| {
+        p.file("src/lib.rs", "pub fn foo() {}")
+            .file(".gitignore", "bar\n")
+    });
 
-    let repo = git2::Repository::init(&p.root()).unwrap();
-    let mut cfg = t!(repo.config());
-    t!(cfg.set_str("user.email", "foo@bar.com"));
-    t!(cfg.set_str("user.name", "Foo Bar"));
-    drop(cfg);
-    git::add(&repo);
-    git::commit(&repo);
     File::create(p.root().join("bar")).unwrap();
 
     p.cargo("fix").run();
@@ -941,7 +905,7 @@ fn both_edition_migrate_flags() {
 error: The argument '--edition' cannot be used with '--prepare-for <prepare-for>'
 
 USAGE:
-    cargo[..] fix --edition --message-format <FMT>
+    cargo[..] fix --edition
 
 For more information try --help
 ";
@@ -1271,8 +1235,7 @@ fn fix_in_existing_repo_weird_ignore() {
             .file("src/lib.rs", "")
             .file(".gitignore", "foo\ninner\n")
             .file("inner/file", "")
-    })
-    .unwrap();
+    });
 
     p.cargo("fix").run();
     // This is questionable about whether it is the right behavior. It should
@@ -1285,3 +1248,50 @@ fn fix_in_existing_repo_weird_ignore() {
         .run();
     p.cargo("fix").cwd("src").run();
 }
+
+#[cargo_test]
+fn fix_with_clippy() {
+    if !is_nightly() {
+        // fix --clippy is unstable
+        eprintln!("skipping test: requires nightly");
+        return;
+    }
+
+    if !clippy_is_available() {
+        return;
+    }
+
+    let p = project()
+        .file(
+            "src/lib.rs",
+            "
+                pub fn foo() {
+                    let mut v = Vec::<String>::new();
+                    let _ = v.iter_mut().filter(|&ref a| a.is_empty());
+                }
+    ",
+        )
+        .build();
+
+    let stderr = "\
+[CHECKING] foo v0.0.1 ([..])
+[FIXING] src/lib.rs (1 fix)
+[FINISHED] [..]
+";
+
+    p.cargo("fix -Zunstable-options --clippy --allow-no-vcs")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(stderr)
+        .with_stdout("")
+        .run();
+
+    assert_eq!(
+        p.read_file("src/lib.rs"),
+        "
+                pub fn foo() {
+                    let mut v = Vec::<String>::new();
+                    let _ = v.iter_mut().filter(|a| a.is_empty());
+                }
+    "
+    );
+}
