@@ -1,18 +1,17 @@
 use std::fs::{self, File};
 use std::io::prelude::*;
 
-use cargo::util::process;
-use cargotest::sleep_ms;
-use cargotest::support::paths::{self, CargoPathExt};
-use cargotest::support::registry::Package;
-use cargotest::support::{execs, main_file, project};
-use hamcrest::{assert_that, existing_file, is_not};
+use cargo_test_support::paths::{self, CargoPathExt};
+use cargo_test_support::registry::Package;
+use cargo_test_support::{basic_lib_manifest, basic_manifest, main_file, project};
+use cargo_test_support::{sleep_ms, t};
 
-#[test]
-#[cfg(not(windows))] // I have no idea why this is failing spuriously on
-                     // Windows, for more info see #3466.
+#[cargo_test]
+// I have no idea why this is failing spuriously on Windows;
+// for more info, see #3466.
+#[cfg(not(windows))]
 fn cargo_compile_with_nested_deps_shorthand() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -58,20 +57,7 @@ fn cargo_compile_with_nested_deps_shorthand() {
             }
         "#,
         )
-        .file(
-            "bar/baz/Cargo.toml",
-            r#"
-            [project]
-
-            name = "baz"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-
-            [lib]
-
-            name = "baz"
-        "#,
-        )
+        .file("bar/baz/Cargo.toml", &basic_lib_manifest("baz"))
         .file(
             "bar/baz/src/baz.rs",
             r#"
@@ -82,59 +68,44 @@ fn cargo_compile_with_nested_deps_shorthand() {
         )
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(&format!(
-            "[COMPILING] baz v0.5.0 ({}/bar/baz)\n\
-             [COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] baz v0.5.0 ([CWD]/bar/baz)\n\
+             [COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url(),
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 
-    assert_that(&p.bin("foo"), existing_file());
+    assert!(p.bin("foo").is_file());
 
-    assert_that(
-        process(&p.bin("foo")),
-        execs().with_stdout("test passed\n").with_status(0),
-    );
+    p.process(&p.bin("foo")).with_stdout("test passed\n").run();
 
     println!("cleaning");
-    assert_that(
-        p.cargo("clean").arg("-v"),
-        execs().with_stdout("").with_status(0),
-    );
+    p.cargo("clean -v").with_stdout("").run();
     println!("building baz");
-    assert_that(
-        p.cargo("build").arg("-p").arg("baz"),
-        execs().with_status(0).with_stderr(&format!(
-            "[COMPILING] baz v0.5.0 ({}/bar/baz)\n\
+    p.cargo("build -p baz")
+        .with_stderr(
+            "[COMPILING] baz v0.5.0 ([CWD]/bar/baz)\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url()
-        )),
-    );
+        )
+        .run();
     println!("building foo");
-    assert_that(
-        p.cargo("build").arg("-p").arg("foo"),
-        execs().with_status(0).with_stderr(&format!(
-            "[COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build -p foo")
+        .with_stderr(
+            "[COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn cargo_compile_with_root_dev_deps() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -155,17 +126,9 @@ fn cargo_compile_with_root_dev_deps() {
         )
         .file("src/main.rs", &main_file(r#""{}", bar::gimme()"#, &["bar"]))
         .build();
-    let _p2 = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-
-            name = "bar"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-        "#,
-        )
+    let _p2 = project()
+        .at("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.5.0"))
         .file(
             "src/lib.rs",
             r#"
@@ -176,12 +139,15 @@ fn cargo_compile_with_root_dev_deps() {
         )
         .build();
 
-    assert_that(p.cargo("build"), execs().with_status(101))
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr_contains("[..]can't find crate for `bar`")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn cargo_compile_with_root_dev_deps_with_testing() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -202,17 +168,9 @@ fn cargo_compile_with_root_dev_deps_with_testing() {
         )
         .file("src/main.rs", &main_file(r#""{}", bar::gimme()"#, &["bar"]))
         .build();
-    let _p2 = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-
-            name = "bar"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-        "#,
-        )
+    let _p2 = project()
+        .at("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.5.0"))
         .file(
             "src/lib.rs",
             r#"
@@ -223,23 +181,21 @@ fn cargo_compile_with_root_dev_deps_with_testing() {
         )
         .build();
 
-    assert_that(
-        p.cargo("test"),
-        execs()
-            .with_stderr(
-                "\
+    p.cargo("test")
+        .with_stderr(
+            "\
 [COMPILING] [..] v0.5.0 ([..])
 [COMPILING] [..] v0.5.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
-[RUNNING] target[/]debug[/]deps[/]foo-[..][EXE]",
-            )
-            .with_stdout_contains("running 0 tests"),
-    );
+[RUNNING] target/debug/deps/foo-[..][EXE]",
+        )
+        .with_stdout_contains("running 0 tests")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn cargo_compile_with_transitive_dev_deps() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -284,26 +240,23 @@ fn cargo_compile_with_transitive_dev_deps() {
         )
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) in \
              [..]\n",
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 
-    assert_that(&p.bin("foo"), existing_file());
+    assert!(p.bin("foo").is_file());
 
-    assert_that(process(&p.bin("foo")), execs().with_stdout("zoidberg\n"));
+    p.process(&p.bin("foo")).with_stdout("zoidberg\n").run();
 }
 
-#[test]
+#[cargo_test]
 fn no_rebuild_dependency() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -317,45 +270,19 @@ fn no_rebuild_dependency() {
             path = "bar"
         "#,
         )
-        .file(
-            "src/main.rs",
-            r#"
-            extern crate bar;
-            fn main() { bar::bar() }
-        "#,
-        )
-        .file(
-            "bar/Cargo.toml",
-            r#"
-            [project]
-
-            name = "bar"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-
-            [lib]
-            name = "bar"
-        "#,
-        )
-        .file(
-            "bar/src/bar.rs",
-            r#"
-            pub fn bar() {}
-        "#,
-        )
+        .file("src/main.rs", "extern crate bar; fn main() { bar::bar() }")
+        .file("bar/Cargo.toml", &basic_lib_manifest("bar"))
+        .file("bar/src/bar.rs", "pub fn bar() {}")
         .build();
     // First time around we should compile both foo and bar
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 
     sleep_ms(1000);
     p.change_file(
@@ -366,20 +293,18 @@ fn no_rebuild_dependency() {
     "#,
     );
     // Don't compile bar, but do recompile foo.
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(
-            "\
-             [COMPILING] foo v0.5.0 ([..])\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] foo v0.5.0 ([..])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-        ),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn deep_dependencies_trigger_rebuild() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -393,13 +318,7 @@ fn deep_dependencies_trigger_rebuild() {
             path = "bar"
         "#,
         )
-        .file(
-            "src/main.rs",
-            r#"
-            extern crate bar;
-            fn main() { bar::bar() }
-        "#,
-        )
+        .file("src/main.rs", "extern crate bar; fn main() { bar::bar() }")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -417,45 +336,21 @@ fn deep_dependencies_trigger_rebuild() {
         )
         .file(
             "bar/src/bar.rs",
-            r#"
-            extern crate baz;
-            pub fn bar() { baz::baz() }
-        "#,
-        )
-        .file(
-            "baz/Cargo.toml",
-            r#"
-            [project]
-
-            name = "baz"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-
-            [lib]
-            name = "baz"
-        "#,
-        )
-        .file(
-            "baz/src/baz.rs",
-            r#"
-            pub fn baz() {}
-        "#,
+            "extern crate baz; pub fn bar() { baz::baz() }",
         )
+        .file("baz/Cargo.toml", &basic_lib_manifest("baz"))
+        .file("baz/src/baz.rs", "pub fn baz() {}")
         .build();
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] baz v0.5.0 ({}/baz)\n\
-             [COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] baz v0.5.0 ([CWD]/baz)\n\
+             [COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url(),
-            p.url(),
-            p.url()
-        )),
-    );
-    assert_that(p.cargo("build"), execs().with_stdout(""));
+        )
+        .run();
+    p.cargo("build").with_stdout("").run();
 
     // Make sure an update to baz triggers a rebuild of bar
     //
@@ -464,25 +359,18 @@ fn deep_dependencies_trigger_rebuild() {
     sleep_ms(1000);
     File::create(&p.root().join("baz/src/baz.rs"))
         .unwrap()
-        .write_all(
-            br#"
-        pub fn baz() { println!("hello!"); }
-    "#,
-        )
+        .write_all(br#"pub fn baz() { println!("hello!"); }"#)
         .unwrap();
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] baz v0.5.0 ({}/baz)\n\
-             [COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    sleep_ms(1000);
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] baz v0.5.0 ([CWD]/baz)\n\
+             [COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url(),
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 
     // Make sure an update to bar doesn't trigger baz
     sleep_ms(1000);
@@ -495,22 +383,20 @@ fn deep_dependencies_trigger_rebuild() {
     "#,
         )
         .unwrap();
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    sleep_ms(1000);
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn no_rebuild_two_deps() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -526,13 +412,7 @@ fn no_rebuild_two_deps() {
             path = "baz"
         "#,
         )
-        .file(
-            "src/main.rs",
-            r#"
-            extern crate bar;
-            fn main() { bar::bar() }
-        "#,
-        )
+        .file("src/main.rs", "extern crate bar; fn main() { bar::bar() }")
         .file(
             "bar/Cargo.toml",
             r#"
@@ -548,53 +428,27 @@ fn no_rebuild_two_deps() {
             path = "../baz"
         "#,
         )
-        .file(
-            "bar/src/bar.rs",
-            r#"
-            pub fn bar() {}
-        "#,
-        )
-        .file(
-            "baz/Cargo.toml",
-            r#"
-            [project]
-
-            name = "baz"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-
-            [lib]
-            name = "baz"
-        "#,
-        )
-        .file(
-            "baz/src/baz.rs",
-            r#"
-            pub fn baz() {}
-        "#,
-        )
+        .file("bar/src/bar.rs", "pub fn bar() {}")
+        .file("baz/Cargo.toml", &basic_lib_manifest("baz"))
+        .file("baz/src/baz.rs", "pub fn baz() {}")
         .build();
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] baz v0.5.0 ({}/baz)\n\
-             [COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] baz v0.5.0 ([CWD]/baz)\n\
+             [COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url(),
-            p.url(),
-            p.url()
-        )),
-    );
-    assert_that(&p.bin("foo"), existing_file());
-    assert_that(p.cargo("build"), execs().with_stdout(""));
-    assert_that(&p.bin("foo"), existing_file());
+        )
+        .run();
+    assert!(p.bin("foo").is_file());
+    p.cargo("build").with_stdout("").run();
+    assert!(p.bin("foo").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn nested_deps_recompile() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -611,61 +465,38 @@ fn nested_deps_recompile() {
         "#,
         )
         .file("src/main.rs", &main_file(r#""{}", bar::gimme()"#, &["bar"]))
-        .file(
-            "src/bar/Cargo.toml",
-            r#"
-            [project]
-
-            name = "bar"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-
-            [lib]
-
-            name = "bar"
-        "#,
-        )
+        .file("src/bar/Cargo.toml", &basic_lib_manifest("bar"))
         .file("src/bar/src/bar.rs", "pub fn gimme() -> i32 { 92 }")
         .build();
-    let bar = p.url();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] bar v0.5.0 ({}/src/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] bar v0.5.0 ([CWD]/src/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            bar,
-            p.url()
-        )),
-    );
+        )
+        .run();
     sleep_ms(1000);
 
     File::create(&p.root().join("src/main.rs"))
         .unwrap()
-        .write_all(
-            br#"
-        fn main() {}
-    "#,
-        )
+        .write_all(br#"fn main() {}"#)
         .unwrap();
 
     // This shouldn't recompile `bar`
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url()
-        )),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn error_message_for_missing_manifest() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -684,38 +515,30 @@ fn error_message_for_missing_manifest() {
         .file("src/bar/not-a-manifest", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to load source for a dependency on `bar`
 
 Caused by:
-  Unable to update file://[..]
+  Unable to update [CWD]/src/bar
 
 Caused by:
-  failed to read `[..]bar[/]Cargo.toml`
+  failed to read `[..]bar/Cargo.toml`
 
 Caused by:
   [..] (os error [..])
 ",
-        ),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn override_relative() {
-    let bar = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-
-            name = "bar"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-        "#,
-        )
+    let bar = project()
+        .at("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.5.0"))
         .file("src/lib.rs", "")
         .build();
 
@@ -725,7 +548,7 @@ fn override_relative() {
         .write_all(br#"paths = ["bar"]"#)
         .unwrap();
 
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             &format!(
@@ -744,36 +567,22 @@ fn override_relative() {
         )
         .file("src/lib.rs", "")
         .build();
-    assert_that(p.cargo("build").arg("-v"), execs().with_status(0));
+    p.cargo("build -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn override_self() {
-    let bar = project("bar")
-        .file(
-            "Cargo.toml",
-            r#"
-            [package]
-
-            name = "bar"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-        "#,
-        )
+    let bar = project()
+        .at("bar")
+        .file("Cargo.toml", &basic_manifest("bar", "0.5.0"))
         .file("src/lib.rs", "")
         .build();
 
-    let p = project("foo");
-    let root = p.root().clone();
-    let p = p.file(
-        ".cargo/config",
-        &format!(
-            r#"
-            paths = ['{}']
-        "#,
-            root.display()
-        ),
-    ).file(
+    let p = project();
+    let root = p.root();
+    let p = p
+        .file(".cargo/config", &format!("paths = ['{}']", root.display()))
+        .file(
             "Cargo.toml",
             &format!(
                 r#"
@@ -794,12 +603,13 @@ fn override_self() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(p.cargo("build"), execs().with_status(0));
+    p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn override_path_dep() {
-    let bar = project("bar")
+    let bar = project()
+        .at("bar")
         .file(
             "p1/Cargo.toml",
             r#"
@@ -813,25 +623,15 @@ fn override_path_dep() {
        "#,
         )
         .file("p1/src/lib.rs", "")
-        .file(
-            "p2/Cargo.toml",
-            r#"
-            [package]
-            name = "p2"
-            version = "0.5.0"
-            authors = []
-       "#,
-        )
+        .file("p2/Cargo.toml", &basic_manifest("p2", "0.5.0"))
         .file("p2/src/lib.rs", "")
         .build();
 
-    let p = project("foo")
+    let p = project()
         .file(
             ".cargo/config",
             &format!(
-                r#"
-            paths = ['{}', '{}']
-        "#,
+                "paths = ['{}', '{}']",
                 bar.root().join("p1").display(),
                 bar.root().join("p2").display()
             ),
@@ -856,12 +656,12 @@ fn override_path_dep() {
         .file("src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build").arg("-v"), execs().with_status(0));
+    p.cargo("build -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn path_dep_build_cmd() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -902,30 +702,22 @@ fn path_dep_build_cmd() {
             }
         "#,
         )
-        .file(
-            "bar/src/bar.rs.in",
-            r#"
-            pub fn gimme() -> i32 { 0 }
-        "#,
-        )
+        .file("bar/src/bar.rs.in", "pub fn gimme() -> i32 { 0 }")
         .build();
     p.root().join("bar").move_into_the_past();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) in \
              [..]\n",
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 
-    assert_that(&p.bin("foo"), existing_file());
+    assert!(p.bin("foo").is_file());
 
-    assert_that(process(&p.bin("foo")), execs().with_stdout("0\n"));
+    p.process(&p.bin("foo")).with_stdout("0\n").run();
 
     // Touching bar.rs.in should cause the `build` command to run again.
     {
@@ -935,24 +727,21 @@ fn path_dep_build_cmd() {
             .unwrap();
     }
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(&format!(
-            "[COMPILING] bar v0.5.0 ({}/bar)\n\
-             [COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .with_stderr(
+            "[COMPILING] bar v0.5.0 ([CWD]/bar)\n\
+             [COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) in \
              [..]\n",
-            p.url(),
-            p.url()
-        )),
-    );
+        )
+        .run();
 
-    assert_that(process(&p.bin("foo")), execs().with_stdout("1\n"));
+    p.process(&p.bin("foo")).with_stdout("1\n").run();
 }
 
-#[test]
+#[cargo_test]
 fn dev_deps_no_rebuild_lib() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -976,47 +765,33 @@ fn dev_deps_no_rebuild_lib() {
             #[cfg(not(test))] pub fn foo() { env!("FOO"); }
         "#,
         )
-        .file(
-            "bar/Cargo.toml",
-            r#"
-            [package]
-
-            name = "bar"
-            version = "0.5.0"
-            authors = ["wycats@example.com"]
-        "#,
-        )
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.5.0"))
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
-    assert_that(
-        p.cargo("build").env("FOO", "bar"),
-        execs().with_status(0).with_stderr(&format!(
-            "[COMPILING] foo v0.5.0 ({})\n\
+    p.cargo("build")
+        .env("FOO", "bar")
+        .with_stderr(
+            "[COMPILING] foo v0.5.0 ([CWD])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) \
              in [..]\n",
-            p.url()
-        )),
-    );
+        )
+        .run();
 
-    assert_that(
-        p.cargo("test"),
-        execs()
-            .with_status(0)
-            .with_stderr(&format!(
-                "\
-[COMPILING] [..] v0.5.0 ({url}[..])
-[COMPILING] [..] v0.5.0 ({url}[..])
+    p.cargo("test")
+        .with_stderr(
+            "\
+[COMPILING] [..] v0.5.0 ([CWD][..])
+[COMPILING] [..] v0.5.0 ([CWD][..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
-[RUNNING] target[/]debug[/]deps[/]foo-[..][EXE]",
-                url = p.url()
-            ))
-            .with_stdout_contains("running 0 tests"),
-    );
+[RUNNING] target/debug/deps/foo-[..][EXE]",
+        )
+        .with_stdout_contains("running 0 tests")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn custom_target_no_rebuild() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -1031,15 +806,7 @@ fn custom_target_no_rebuild() {
         "#,
         )
         .file("src/lib.rs", "")
-        .file(
-            "a/Cargo.toml",
-            r#"
-            [project]
-            name = "a"
-            version = "0.5.0"
-            authors = []
-        "#,
-        )
+        .file("a/Cargo.toml", &basic_manifest("a", "0.5.0"))
         .file("a/src/lib.rs", "")
         .file(
             "b/Cargo.toml",
@@ -1054,37 +821,35 @@ fn custom_target_no_rebuild() {
         )
         .file("b/src/lib.rs", "")
         .build();
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
 [COMPILING] a v0.5.0 ([..])
 [COMPILING] foo v0.5.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ),
-    );
+        )
+        .run();
 
     t!(fs::rename(
         p.root().join("target"),
         p.root().join("target_moved")
     ));
-    assert_that(
-        p.cargo("build")
-            .arg("--manifest-path=b/Cargo.toml")
-            .env("CARGO_TARGET_DIR", "target_moved"),
-        execs().with_status(0).with_stderr(
+    p.cargo("build --manifest-path=b/Cargo.toml")
+        .env("CARGO_TARGET_DIR", "target_moved")
+        .with_stderr(
             "\
 [COMPILING] b v0.5.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn override_and_depend() {
-    let p = project("foo")
+    let p = project()
+        .no_manifest()
         .file(
             "a/a1/Cargo.toml",
             r#"
@@ -1097,15 +862,7 @@ fn override_and_depend() {
         "#,
         )
         .file("a/a1/src/lib.rs", "")
-        .file(
-            "a/a2/Cargo.toml",
-            r#"
-            [project]
-            name = "a2"
-            version = "0.5.0"
-            authors = []
-        "#,
-        )
+        .file("a/a2/Cargo.toml", &basic_manifest("a2", "0.5.0"))
         .file("a/a2/src/lib.rs", "")
         .file(
             "b/Cargo.toml",
@@ -1120,49 +877,34 @@ fn override_and_depend() {
         "#,
         )
         .file("b/src/lib.rs", "")
-        .file(
-            "b/.cargo/config",
-            r#"
-            paths = ["../a"]
-        "#,
-        )
+        .file("b/.cargo/config", r#"paths = ["../a"]"#)
         .build();
-    assert_that(
-        p.cargo("build").cwd(p.root().join("b")),
-        execs().with_status(0).with_stderr(
+    p.cargo("build")
+        .cwd("b")
+        .with_stderr(
             "\
 [COMPILING] a2 v0.5.0 ([..])
 [COMPILING] a1 v0.5.0 ([..])
 [COMPILING] b v0.5.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn missing_path_dependency() {
-    let p = project("foo")
-        .file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "a"
-            version = "0.5.0"
-            authors = []
-        "#,
-        )
+    let p = project()
+        .file("Cargo.toml", &basic_manifest("a", "0.5.0"))
         .file("src/lib.rs", "")
         .file(
             ".cargo/config",
-            r#"
-            paths = ["../whoa-this-does-not-exist"]
-        "#,
+            r#"paths = ["../whoa-this-does-not-exist"]"#,
         )
         .build();
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to update path override `[..]../whoa-this-does-not-exist` \
 (defined in `[..]`)
@@ -1173,15 +915,15 @@ Caused by:
 Caused by:
   [..] (os error [..])
 ",
-        ),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn invalid_path_dep_in_workspace_with_lockfile() {
     Package::new("bar", "1.0.0").publish();
 
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -1213,7 +955,7 @@ fn invalid_path_dep_in_workspace_with_lockfile() {
         .build();
 
     // Generate a lock file
-    assert_that(p.cargo("build"), execs().with_status(0));
+    p.cargo("build").run();
 
     // Change the dependency on `bar` to an invalid path
     File::create(&p.root().join("foo/Cargo.toml"))
@@ -1233,21 +975,22 @@ fn invalid_path_dep_in_workspace_with_lockfile() {
 
     // Make sure we get a nice error. In the past this actually stack
     // overflowed!
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: no matching package named `bar` found
 location searched: [..]
+perhaps you meant: foo
 required by package `foo v0.5.0 ([..])`
 ",
-        ),
-    );
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn workspace_produces_rlib() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -1263,30 +1006,19 @@ fn workspace_produces_rlib() {
         "#,
         )
         .file("src/lib.rs", "")
-        .file(
-            "foo/Cargo.toml",
-            r#"
-            [project]
-            name = "foo"
-            version = "0.5.0"
-            authors = []
-        "#,
-        )
+        .file("foo/Cargo.toml", &basic_manifest("foo", "0.5.0"))
         .file("foo/src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build"), execs().with_status(0));
+    p.cargo("build").run();
 
-    assert_that(&p.root().join("target/debug/libtop.rlib"), existing_file());
-    assert_that(
-        &p.root().join("target/debug/libfoo.rlib"),
-        is_not(existing_file()),
-    );
+    assert!(p.root().join("target/debug/libtop.rlib").is_file());
+    assert!(!p.root().join("target/debug/libfoo.rlib").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn thin_lto_works() {
-    let p = project("foo")
+    let p = project()
         .file(
             "Cargo.toml",
             r#"
@@ -1302,14 +1034,13 @@ fn thin_lto_works() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--release").arg("-v"),
-        execs().with_stderr(
+    p.cargo("build --release -v")
+        .with_stderr(
             "\
 [COMPILING] top [..]
 [RUNNING] `rustc [..] -C lto=thin [..]`
 [FINISHED] [..]
 ",
-        ),
-    );
+        )
+        .run();
 }
