@@ -14,11 +14,11 @@
 
 use if_chain::if_chain;
 use rustc::lint::{EarlyContext, EarlyLintPass, LintArray, LintPass};
-use rustc::{declare_tool_lint, lint_array};
+use rustc::{declare_lint_pass, declare_tool_lint};
 use syntax::ast;
 
 use crate::utils::sugg::Sugg;
-use crate::utils::{in_macro, snippet_block, snippet_block_with_applicability, span_lint_and_sugg, span_lint_and_then};
+use crate::utils::{snippet_block, snippet_block_with_applicability, span_lint_and_sugg, span_lint_and_then};
 use rustc_errors::Applicability;
 
 declare_clippy_lint! {
@@ -71,40 +71,25 @@ declare_clippy_lint! {
     "`if`s that can be collapsed (e.g., `if x { if y { ... } }` and `else { if x { ... } }`)"
 }
 
-#[derive(Copy, Clone)]
-pub struct CollapsibleIf;
-
-impl LintPass for CollapsibleIf {
-    fn get_lints(&self) -> LintArray {
-        lint_array!(COLLAPSIBLE_IF)
-    }
-
-    fn name(&self) -> &'static str {
-        "CollapsibleIf"
-    }
-}
+declare_lint_pass!(CollapsibleIf => [COLLAPSIBLE_IF]);
 
 impl EarlyLintPass for CollapsibleIf {
     fn check_expr(&mut self, cx: &EarlyContext<'_>, expr: &ast::Expr) {
-        if !in_macro(expr.span) {
+        if !expr.span.from_expansion() {
             check_if(cx, expr)
         }
     }
 }
 
 fn check_if(cx: &EarlyContext<'_>, expr: &ast::Expr) {
-    match expr.node {
-        ast::ExprKind::If(ref check, ref then, ref else_) => {
-            if let Some(ref else_) = *else_ {
-                check_collapsible_maybe_if_let(cx, else_);
-            } else {
-                check_collapsible_no_if_let(cx, expr, check, then);
-            }
-        },
-        ast::ExprKind::IfLet(_, _, _, Some(ref else_)) => {
+    if let ast::ExprKind::If(check, then, else_) = &expr.node {
+        if let Some(else_) = else_ {
             check_collapsible_maybe_if_let(cx, else_);
-        },
-        _ => (),
+        } else if let ast::ExprKind::Let(..) = check.node {
+            // Prevent triggering on `if let a = b { if c { .. } }`.
+        } else {
+            check_collapsible_no_if_let(cx, expr, check, then);
+        }
     }
 }
 
@@ -121,23 +106,19 @@ fn check_collapsible_maybe_if_let(cx: &EarlyContext<'_>, else_: &ast::Expr) {
         if let ast::ExprKind::Block(ref block, _) = else_.node;
         if !block_starts_with_comment(cx, block);
         if let Some(else_) = expr_block(block);
-        if !in_macro(else_.span);
+        if !else_.span.from_expansion();
+        if let ast::ExprKind::If(..) = else_.node;
         then {
-            match else_.node {
-                ast::ExprKind::If(..) | ast::ExprKind::IfLet(..) => {
-                    let mut applicability = Applicability::MachineApplicable;
-                    span_lint_and_sugg(
-                        cx,
-                        COLLAPSIBLE_IF,
-                        block.span,
-                        "this `else { if .. }` block can be collapsed",
-                        "try",
-                        snippet_block_with_applicability(cx, else_.span, "..", &mut applicability).into_owned(),
-                        applicability,
-                    );
-                }
-                _ => (),
-            }
+            let mut applicability = Applicability::MachineApplicable;
+            span_lint_and_sugg(
+                cx,
+                COLLAPSIBLE_IF,
+                block.span,
+                "this `else { if .. }` block can be collapsed",
+                "try",
+                snippet_block_with_applicability(cx, else_.span, "..", &mut applicability).into_owned(),
+                applicability,
+            );
         }
     }
 }
@@ -148,6 +129,11 @@ fn check_collapsible_no_if_let(cx: &EarlyContext<'_>, expr: &ast::Expr, check: &
         if let Some(inner) = expr_block(then);
         if let ast::ExprKind::If(ref check_inner, ref content, None) = inner.node;
         then {
+            if let ast::ExprKind::Let(..) = check_inner.node {
+                // Prevent triggering on `if c { if let a = b { .. } }`.
+                return;
+            }
+
             if expr.span.ctxt() != inner.span.ctxt() {
                 return;
             }
